require 'logger'

module Enc
  module Utils
    module Logging
      def logger
        unless @logger
          @logger = Logging.logger_for(self.class.name)
        end
        @logger
      end

      @loggers = {}

      class << self
        def logger_for(classname)
          unless @loggers.has_key?(classname)
            set_config
            set_log_level
            set_log_file
            @loggers[classname] = configure_logger(classname)
          end
          @loggers[classname]
        end

        def configure_logger(classname)
          logger = Logger.new(@file)
          logger.level = @level
          logger.progname = classname
          logger
        end

        def set_config
          unless @config
            @config = Enc::Config.new
          end
          @config
        end

        def set_log_level
          unless @level
            @level = convert_log_level(@config.get('jive_enc_log_level'))
          end
          @level
        end

        def set_log_file
          unless @file
            @file = @config.get('jive_enc_log_file')
          end
        end

        def convert_log_level(level)
          case level
            when 'warn'
              Logger::WARN
            when 'error'
              Logger::ERROR
            when 'debug'
              Logger::DEBUG
            else
              Logger::INFO
          end
        end
      end
    end
  end
end